@@ -80,7 +80,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       }
       
       if(sheetPr.size() == 0)
-        sheetPr = getChildlessNode(xml_pre, "<sheetPr");
+        sheetPr = getChildlessNode(xml_pre, "sheetPr");
       
       if(sheetPr.size() > 0)
         this_worksheet.field("sheetPr") = sheetPr;
@@ -88,7 +88,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       
       
       // Freeze Panes
-      CharacterVector node_xml = getChildlessNode(xml_pre, "<pane ");
+      CharacterVector node_xml = getChildlessNode(xml_pre, "pane");
       if(node_xml.size() > 0)
         this_worksheet.field("freezePane") = node_xml;
       
@@ -211,28 +211,28 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       
       std::string xml_post = xml.substr(pos_post);
       
-      node_xml = getChildlessNode(xml_post, "<sheetProtection ");
+      node_xml = getChildlessNode(xml_post, "sheetProtection");
       if(node_xml.size() > 0) {
         this_worksheet.field("sheetProtection") = node_xml;
       }
       
       
-      node_xml = getChildlessNode(xml_post, "<autoFilter ");
+      node_xml = getChildlessNode(xml_post, "autoFilter");
       if(node_xml.size() > 0)
         this_worksheet.field("autoFilter") = node_xml;
       
       
-      node_xml = getChildlessNode(xml_post, "<hyperlink ");
+      node_xml = getChildlessNode(xml_post, "hyperlink");
       if(node_xml.size() > 0)
         this_worksheet.field("hyperlinks") = node_xml;
       
       
-      node_xml = getChildlessNode(xml_post, "<pageMargins ");
+      node_xml = getChildlessNode(xml_post, "pageMargins");
       if(node_xml.size() > 0)
         this_worksheet.field("pageMargins") = node_xml;
       
       
-      node_xml = getChildlessNode(xml_post, "<pageSetup ");
+      node_xml = getChildlessNode(xml_post, "pageSetup");
       if(node_xml.size() > 0){
         for(int j = 0; j < node_xml.size(); j++){
           
@@ -252,7 +252,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
         this_worksheet.field("pageSetup") = node_xml;
       }
       
-      node_xml = getChildlessNode(xml_post, "<mergeCell ");
+      node_xml = getChildlessNode(xml_post, "mergeCell");
       if(node_xml.size() > 0)
         this_worksheet.field("mergeCells") = node_xml;
       
@@ -297,9 +297,9 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       }
       
       
-      node_xml = getChildlessNode(xml_post, "<drawing ");
+      node_xml = getChildlessNode(xml_post, "drawing");
       if(node_xml.size() == 0)
-        node_xml = getChildlessNode(xml_post, "<legacyDrawing ");
+        node_xml = getChildlessNode(xml_post, "legacyDrawing");
       
       if(node_xml.size() > 0){
         for(int j = 0; j < node_xml.size(); j++){
@@ -472,6 +472,14 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
               has_v = true;
             }
             
+            // find <is><t> tag and </t></is> end tag
+            endPos = cell.find("</t></is>", 0);
+            if(endPos != std::string::npos){
+              pos = cell.find("<is><t", 0);
+              pos = cell.find(">", pos);
+              v[j] = cell.substr(pos + 4, endPos - pos - 4); // skip <t> and </t
+              has_v = true;
+            }
             
             // Pull out type
             pos_t = cell.find(" t=", 0);
@@ -778,8 +786,8 @@ SEXP getNodes(std::string xml, std::string tagIn){
     
   }  
   
-  return wrap(r) ;  
-  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
 }
 
 
@@ -814,9 +822,8 @@ SEXP getOpenClosedNode(std::string xml, std::string open_tag, std::string close_
     
   }  
   
-  return wrap(r) ;  
-  
-  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
 }
 
 
@@ -852,7 +859,7 @@ SEXP getAttr(CharacterVector x, std::string tag){
     }
   }
   
-  return wrap(r) ;  
+  return markUTF8(r);   // no need to wrap as r is already a CharacterVector
   
 }
 
@@ -887,38 +894,93 @@ std::vector<std::string> getChildlessNode_ss(std::string xml, std::string tag){
 
 
 // [[Rcpp::export]]
-CharacterVector getChildlessNode(std::string xml, std::string tag){
-  
-  size_t k = tag.length();
+CharacterVector getChildlessNode(std::string xml, std::string tag) {
+
+  // size_t k = tag.length(); variable not used
   if(xml.length() == 0)
     return wrap(NA_STRING);
-  
-  xml = " " + xml;
-  
+
+  size_t begPos = 0, endPos = 0;
+
   std::vector<std::string> r;
-  size_t pos = 0;
-  size_t endPos = 0;
-  std::string tagEnd = "/>";
-  
-  while(1){
-    
-    pos = xml.find(tag, pos+1);    
-    if(pos == std::string::npos)
-      break;
-    
-    endPos = xml.find(tagEnd, pos+k);
-    
-    r.push_back(xml.substr(pos, endPos-pos+2).c_str());
-    
+  std::string res = "";
+
+  // check "<tag "
+  const std::string begTag = "<" + tag;
+  const std::string endTag = ">";
+  std::string temp_endTag = "";
+
+  // initial check, which kind of tags to expect
+  begPos = xml.find(begTag, begPos);
+
+  // if begTag was found
+  // try with <foo ... />
+  while( begPos != std::string::npos ) {
+
+    endPos = xml.find(endTag, begPos);
+    if(begPos == std::string::npos || endPos == std::string::npos) break;
+    res = xml.substr(begPos, (endPos - begPos) + endTag.length());
+    if (res.length() == 0) break;
+
+    auto itr = 0;
+    // check if we have either <foo ...>, <foo/>, or <foo>. We have to avoid
+    // <foos...>
+    while (
+        res.substr(begTag.length(),1).compare(" ") != 0 &&   // <foo ...>
+          res.substr(begTag.length(),1).compare("/") != 0 && // <foo/>
+          res.substr(begTag.length(),1).compare(">") != 0    // <foo>
+    ) {
+      
+      if (itr == 0) begPos = begPos + begTag.length();
+      if(begPos == std::string::npos || endPos == std::string::npos) break;
+
+      Rcpp::checkUserInterrupt();
+
+      begPos = xml.find(begTag, begPos);
+      endPos = xml.find(endTag, begPos);
+
+      if(begPos == std::string::npos || endPos == std::string::npos) break;
+      res = xml.substr(begPos, (endPos - begPos) + endTag.length());
+
+      ++itr;
+    }
+
+    // if we have <foo> we need to find the matching closing tag </foo>
+    bool closingtag = false;
+    if (res.substr( res.length() - 2 ).compare("/>") != 0) {
+      // this node has </tag>
+      temp_endTag = "</" + tag + ">";
+      closingtag = true;
+    } else {
+      temp_endTag = endTag;
+    }
+
+    // if we have a closing tag, we need to reposition the endPos. Previously
+    // it was at the end of <foo>. Now we search for </foo>
+    if (closingtag) {
+      endPos = xml.find(temp_endTag, begPos);
+      if(begPos == std::string::npos || endPos == std::string::npos) break;
+      
+      // read from initial "<" to final ">"
+      res = xml.substr(begPos, (endPos - begPos) + temp_endTag.length());
+      if (res.length() == 0) break;
+    }
+
+    if(begPos == std::string::npos || endPos == std::string::npos) break;
+
+    begPos = endPos + temp_endTag.length();
+    begPos = xml.find(begTag, begPos);
+    r.push_back(res);
+
+    if(begPos == std::string::npos || endPos == std::string::npos) break;
   }
-  
-  return wrap(r) ;  
-  
+
+  CharacterVector out = wrap(r);
+  return markUTF8(out);
+
 }
 
 
-
-
 // [[Rcpp::export]]
 CharacterVector get_extLst_Major(std::string xml){
   
@@ -960,7 +1022,8 @@ CharacterVector get_extLst_Major(std::string xml){
     
   }
   
-  return wrap(r) ;  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
   
 }
 
